﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEditor;
using System.IO;
using System;
using System.Text;

public class FntPostProcessor : AssetPostprocessor 
{
    private static void OnPostprocessAllAssets(string[] importedAssets, string[] deletedAssets, string[] movedAssets, string[] movedFromPath)
    {
        foreach(string asset in importedAssets) {
            
            if(Path.GetExtension(asset) == ".fnt") {
                Debug.Log("Processing: " + asset);
                TextAsset textAsset = AssetDatabase.LoadAssetAtPath(asset, typeof(TextAsset)) as TextAsset;
                GenerateFont(textAsset);
            }
        }
    }
    
    public static void GenerateFont(TextAsset asset)
    {
        string rootPath = Path.GetDirectoryName(AssetDatabase.GetAssetPath(asset));
 
        Texture2D texture = AssetDatabase.LoadAssetAtPath(rootPath + "/" + asset.name + ".png", typeof(Texture2D)) as Texture2D;
        if (!texture) 
            throw new UnityException("Texture2d asset doesn't exist for " + asset.name);
 
        string exportPath = rootPath + "/" + Path.GetFileNameWithoutExtension(asset.name);
        
        Hashtable table = LoadFntFile(AssetDatabase.GetAssetPath(asset));
        
        Debug.Log(table);
        
        if(table != null) {
            GenerateFontSettingsFromFntTable(table, texture, exportPath);
        }
    }
  
    private static List<string> TokenizeFntLine(string line)
    {
        bool inQuote = false;
        string token = "";
        int index = 0;
        List<string> tokens = new List<String>();
        
        while(index < line.Length) {
            
            if(line[index] == '"') {
                inQuote = !inQuote;
                token += line[index];
            }
            else if(line[index] == ' ' && !inQuote) {
                
                // do not add empty token
                if(token.Length != 0) {
                    tokens.Add(token);
                    token = "";
                }
            }
            else {
                token += line[index];
            }
            
            index++;
        }
        
        return tokens;
    }

    private static Hashtable ParseFntLineAsTable(List<string> tokens)
    {
        Hashtable table = new Hashtable();
        
        for(int i=0; i<tokens.Count; i++)
        {
            if(i == 0) {
                table.Add("__lineKey", tokens[i]);
            }
            else {
                string[] pair = tokens[i].Split('=');
                
                string key = pair[0];
                string valStr = pair[1];
                
                if(valStr.StartsWith("\"") ) {
                    table.Add(key, valStr.Replace("\"", ""));
                }
                else if(key == "padding" || key == "spacing") {
                    // ignore padding and spacing since unity does not support them
                    table.Add(key, valStr);
                }
                else {               
                    table.Add(key, Convert.ToInt32(valStr));
                }
            }
        }
        
        return table;
    }
  
    private static Hashtable LoadFntFile(string fileName)
    {
        StreamReader reader = new StreamReader(fileName, Encoding.Default);
        
        using (reader)
        {
            Hashtable fileTable = new Hashtable();
            fileTable.Add("char", new List<Hashtable>());
            fileTable.Add("kerning", new List<Hashtable>());
            
            string line;
            
            // While there's lines left in the text file, do this:
            do
            {
                line = reader.ReadLine();
                if (line != null && line.Length != 0)
                {
                    List<String> tokens = TokenizeFntLine(line);
                    
                    Hashtable item = ParseFntLineAsTable(tokens);
                    
                    if(item["__lineKey"] as string == "char") {
                        (fileTable["char"] as List<Hashtable>).Add(item);
                    }
                    else if(item["__lineKey"] as string == "kerning") {
                        (fileTable["kerning"] as List<Hashtable>).Add(item);
                    }
                    else {
                        fileTable[item["__lineKey"]] = item;
                    }
                }
            }
            while (line != null);
            
            // Done reading, close the reader and return true to broadcast success    
            reader.Close();
            
            return fileTable;
        }
    }
 
    private static int ToInt(Hashtable t, string key)
    {
        return Convert.ToInt32(t[key]);
    }
     
    private static void GenerateFontSettingsFromFntTable(Hashtable table, Texture2D tex, string exportPath)
    {
        List<Hashtable> charList = table["char"] as List<Hashtable>;
        
        string fontName = (table["info"] as Hashtable)["face"] as string;
        int fontSize = ToInt((table["info"] as Hashtable), "size");

        CharacterInfo[] charInfos = new CharacterInfo[charList.Count];
        
        for(int i=0; i<charInfos.Length; i++)
        {
            CharacterInfo charInfo = new CharacterInfo();
         
            Hashtable charTable = charList[i] as Hashtable;            
         
            charInfo.index = Convert.ToInt32(charTable["id"]);
            charInfo.advance = Convert.ToInt32(charTable["xadvance"]);
                     
            // compute uvs
            float left = ((float) ToInt(charTable, "x")) / tex.width;
            float top = 1.0f - (float)ToInt(charTable, "y") / tex.height;
            float width = ((float) ToInt(charTable, "width")) / tex.width;
            float height = ((float) ToInt(charTable, "height")) / tex.height;

            charInfo.uvBottomLeft = new Vector2(left, top-height);
            charInfo.uvBottomRight = new Vector2(left +width, top-height);
            charInfo.uvTopLeft = new Vector2(left, top);
            charInfo.uvTopRight = new Vector2(left+width, top);
            
            int eleft = ToInt(charTable, "xoffset");
            int etop = -ToInt(charTable, "yoffset");
            int ewidth = ToInt(charTable, "width");
            int eheight = ToInt(charTable, "height");
            charInfo.minX = eleft;
            charInfo.minY = etop-eheight;
            charInfo.maxX = eleft+ewidth;
            charInfo.maxY = etop;            
         
            //charInfo.size = fontSize;                        
            charInfo.style = FontStyle.Normal;
            charInfos[i] = charInfo;
        }
        
        // Create material
        Shader shader = Shader.Find("GUI/Text Shader");
        Material material = new Material(shader);
        material.mainTexture = tex;
        AssetDatabase.CreateAsset(material, exportPath + ".mat");
 
        bool newAsset = false;
        Font font = AssetDatabase.LoadAssetAtPath(exportPath + ".fontsettings", typeof(Font)) as Font;
        if(font == null) {
            font = new Font();
            newAsset = true;
        }
        
        // Create font
        font.material = material;
        font.name = fontName;
        font.characterInfo = charInfos;
                
        if(newAsset)
            AssetDatabase.CreateAsset(font, exportPath + ".fontsettings");
        else
            AssetDatabase.SaveAssets();
    }
}
