﻿using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using UnityEditor;
using System.IO;
using System;
using System.Text;

// Allow mesh filter to set mesh pivot point
[CustomEditor(typeof(CubeBase))]
[CanEditMultipleObjects]
public class CubeBaseEditor : Editor
{
	SerializedProperty pivotProperty;
	SerializedProperty alignProperty;

	void OnEnable()
	{
		pivotProperty = serializedObject.FindProperty("pivot");

		foreach(CubeBase o in this.targets) {
			
			pivotProperty.vector3Value = GetPivot(o.transform);
			serializedObject.ApplyModifiedProperties();
			serializedObject.Update();
		}

		alignProperty = serializedObject.FindProperty("alignToGrid");
	}
	
	Transform FindRenderObject(Transform o)
	{
		if(o.GetComponentInChildren<MeshFilter>() != null)
			return o.GetComponentInChildren<MeshFilter>().transform;
		else if(o.GetComponentInChildren<SkinnedMeshRenderer>() != null)
			return o.GetComponentInChildren<SkinnedMeshRenderer>().transform;
		return null;
	}
	
	Mesh FindMesh(Transform o)
	{
		if(o.GetComponentInChildren<MeshFilter>() != null)
			return o.GetComponentInChildren<MeshFilter>().sharedMesh;
		else if(o.GetComponentInChildren<SkinnedMeshRenderer>() != null)
			return o.GetComponentInChildren<SkinnedMeshRenderer>().sharedMesh;
		return null;
	}

	public override void OnInspectorGUI()
	{
		serializedObject.Update();

		EditorGUI.BeginChangeCheck();

		EditorGUILayout.PropertyField(pivotProperty);

		if(EditorGUI.EndChangeCheck()) {
			ApplyPivot(pivotProperty.vector3Value);
			serializedObject.ApplyModifiedProperties();	
		}

		EditorGUI.BeginChangeCheck();
		EditorGUILayout.PropertyField(alignProperty);
		if(EditorGUI.EndChangeCheck()) {
			serializedObject.ApplyModifiedProperties();
		}
	}

	Vector3 GetPivot(Transform root)
	{
		Transform child = FindRenderObject(root);
		Mesh mesh = FindMesh(child);
		
		Vector3 size = mesh.bounds.extents;
		Vector3 pivot = ((-child.localPosition).DivBy(size) + Vector3.one) * 0.5f;
		return pivot;
	}

	void ApplyPivot(Vector3 pivot) 
	{
		foreach(CubeBase o in this.targets) {
			
			Transform child = FindRenderObject(o.transform);
			Mesh mesh = FindMesh(child);
			
			if(mesh != null) {
				Vector3 size = mesh.bounds.extents;
				Vector3 offset = Vector3.Scale(size, -(pivotProperty.vector3Value  * 2.0f - Vector3.one));
				child.localPosition = offset;
			}
		}
	}

	void OnSceneGUI() 
	{
		if(alignProperty.boolValue)
			AlignTiles();
	}

	void AlignTiles()
	{
		Transform o = (this.target as CubeBase).transform;
		o.localPosition = new Vector3(Mathf.RoundToInt(o.localPosition.x), Mathf.RoundToInt(o.localPosition.y), Mathf.RoundToInt (o.localPosition.z));
	}
	
}


public class ObjFormatFix : AssetPostprocessor
{
	public void OnPreprocessModel()
	{
		if(Path.GetExtension(assetPath) == ".obj") {

			string name = Path.GetFileNameWithoutExtension(assetPath);

			ModelImporter modelImporter = (ModelImporter) assetImporter;
			modelImporter.globalScale = 1.0f / 15.0f;
		}
	}

	public void OnPostprocessModel(GameObject o)
	{
		if(Path.GetExtension(assetPath) == ".obj") {

			Debug.Log ("post processing asset " + assetPath);

			// add mesh filter
			Mesh mesh = o.transform.FindChild("default").GetComponent<MeshFilter>().sharedMesh;

			string name = Path.GetFileNameWithoutExtension(assetPath);

			// create material
			Material mat = new Material(Shader.Find("Mobile/Diffuse"));
			AssetDatabase.CreateAsset(mat, "Assets/Models/Materials/"+name+".mat");

			// assign texture
			string texPath = Path.ChangeExtension(assetPath, "png");
			mat.mainTexture = AssetDatabase.LoadAssetAtPath<Texture>(texPath);

			o.transform.FindChild("default").GetComponent<MeshRenderer>().sharedMaterial = mat;

			// add collision body
			// make sure it's the last thing, since it automatically 
			// calculutes a reasonably collision box
			BoxCollider collider = o.AddComponent<BoxCollider>();
		}
	}
}
	
public class MenuItems
{
	[MenuItem("Tools/Generate Tile Prefabs")]
	private static void GenerateTilePrefabs()
	{
		//UnityEditor.EditorUtility.DisplayProgressBar("Tiles", "Generating tiles", 0.0f);

		// create folder if not exsit
		if(!AssetDatabase.IsValidFolder("Assets/Resources/AutoTiles"))
			AssetDatabase.CreateFolder("Assets/Resources", "AutoTiles");

		string[] subfolders = { "characters", "collections", "enemies", "gift", "grasses", "mechanisms", "obstacles", "trees"};

		foreach(string subfolder in subfolders) {
			GenerateTilePrefabsForSubfolder(subfolder);
		}

		AssetDatabase.Refresh();
		UnityEditor.EditorUtility.ClearProgressBar();
	}

	private static void GenerateTilePrefabsForSubfolder(string subfolder)
	{
		if(!AssetDatabase.IsValidFolder("Assets/Resources/AutoTiles/"+subfolder))
			AssetDatabase.CreateFolder("Assets/Resources/AutoTiles", subfolder);

		// clear folder
		foreach(string p in Directory.GetFiles("Assets/Resources/AutoTiles/"+subfolder))
			AssetDatabase.DeleteAsset(p);

		string[] files = Directory.GetFiles("Assets/Models/"+subfolder);

		// generate objects
		foreach(string file in files) {

			if( Path.GetExtension(file) == ".obj") {

				GameObject o = GameObject.Instantiate( AssetDatabase.LoadAssetAtPath<GameObject>(file) );
				o.name = Path.GetFileNameWithoutExtension(file);

				// copy material
				MeshRenderer renderer = o.transform.FindChild("default").GetComponent<MeshRenderer>();
				Material mat = Material.Instantiate( renderer.sharedMaterial );
				mat.name = Path.GetFileNameWithoutExtension(file);
				AssetDatabase.CreateAsset(mat, "Assets/Resources/AutoTiles/" + subfolder + "/" + mat.name + ".mat");

				renderer.sharedMaterial = mat;
				o.AddComponent<CubeBase>();

				PrefabUtility.CreatePrefab("Assets/Resources/AutoTiles/"+ subfolder + "/" + o.name+".prefab", o);
				GameObject.DestroyImmediate(o);
			}
		}
	}
}
