using UnityEngine;
using System.Collections.Generic;
using DG.Tweening;

public struct IntVec2 
{
	public int x;
	public int y;
	
	public IntVec2(int xx, int yy) { x = xx; y = yy; }

	public int RectDistance(IntVec2 b) 
	{
		return Mathf.Abs(b.x - x) + Mathf.Abs(b.y - y);
	}

	public override string ToString() { return x + ", " + y; }

	public static bool operator == (IntVec2 a, IntVec2 b) { return a.Equals(b); }
	public static bool operator != (IntVec2 a, IntVec2 b) { return !a.Equals(b); }
	
	public static IntVec2 operator +(IntVec2 a, IntVec2 b) { return new IntVec2(a.x + b.x, a.y + b.y); }
	public static IntVec2 operator -(IntVec2 a, IntVec2 b) { return new IntVec2(a.x - b.x, a.y - b.y); }
	
	public Vector3 ToVector3(float yIn3D = 0.0f) {
		return new Vector3(x, yIn3D, y);
	}
	
	// override object.Equals
	public override bool Equals (object obj)
	{
		if (obj == null || GetType() != obj.GetType())
		{
			return false;
		}
		
		return base.Equals (obj);
	}
	
	// override object.GetHashCode
	public override int GetHashCode()
	{
		return base.GetHashCode();
	}
}

public class KeyProb<T>
{
	public T key;
	public float prob;
	
	public KeyProb(T k, float v) {
		key = k;
		prob = v;
	}
	
	public static List<KeyProb<T>> ConvertToAdditiveProbList(List<KeyProb<T>> probList) 
	{
		List<KeyProb<T>> additiveList = new List<KeyProb<T>>();
		
		float acc = 0.0f;
		foreach(KeyProb<T> p in probList) {
			KeyProb<T> pair = p;
			acc += pair.prob;
			pair.prob = acc;
			additiveList.Add(pair);
		}
		
		// normalize probability
		for(int i=0; i<additiveList.Count; i++) {
			KeyProb<T> pair = additiveList[i];
			pair.prob = pair.prob / acc;
			additiveList[i] = pair;
		}
		
		return additiveList;
	}
	
	public static T FetchKeyByProb(List<KeyProb<T>> table, float val)
	{
		foreach(KeyProb<T> entry in table) {
			if(val < entry.prob)
				return entry.key;
		}
		
		return default(T);
	}
}


public static class Utils
{	
	//-----------------------------
	// Vector3 extension
	public static IntVec2 RoundToInt(this Vector3 v)
	{
		return new IntVec2( Mathf.RoundToInt(v.x), Mathf.RoundToInt(v.z));
	}
	
	public static Vector3 DivBy(this Vector3 v, Vector3 b)
	{
		return new Vector3(v.x/b.x, v.y/b.y, v.z/b.z);
	}
	
	//-----------------------------
	// Transform extension
	public static void CopyFrom(this Transform self, Transform src)
	{
		self.position = src.position;
		self.rotation = src.rotation;
		self.localScale = src.localScale;
	}
	
	public static void SetLayersRecursively(this Transform trans, string name)
	{
		trans.gameObject.layer = LayerMask.NameToLayer(name);
		foreach(Transform child in trans)
		{            
			child.SetLayersRecursively(name);
		}
	}
	
	public static void SetX(this Transform v, float val) { v.position = new Vector3(val, v.position.y, v.position.z); }
	public static void SetY(this Transform v, float val) { v.position = new Vector3(v.position.x, val, v.position.z); }
	public static void SetZ(this Transform v, float val) { v.position = new Vector3(v.position.x, v.position.y, val); }
	
	public static Vector3 withX(this Vector3 v, float val) { return new Vector3(val, v.y, v.z); }
	public static Vector3 withY(this Vector3 v, float val) { return new Vector3(v.x, val, v.z); }
	public static Vector3 withZ(this Vector3 v, float val) { return new Vector3(v.x, v.y, val); }
	
	//-----------------------------
	// Tweening
	public static Tween Make(this Sequence seq, params Tween[] tweens)
	{		
		foreach(Tween t in tweens)
			seq.Append(t);
		return seq;
	}
	
	// misc methods
	public static void DumpMemoryStats()
	{
		#if ENABLE_PROFILER
		long bytes = 0;
		
		var textures = Resources.FindObjectsOfTypeAll(typeof(Texture));
		foreach(Texture t in textures) {
			bytes += Profiler.GetRuntimeMemorySize(t);
		}
		
		var meshes = Resources.FindObjectsOfTypeAll(typeof(Mesh));
		foreach(Mesh m in meshes) {
			bytes += Profiler.GetRuntimeMemorySize(m);
		}
		
		var mats = Resources.FindObjectsOfTypeAll(typeof(Material));
		foreach(Material m in mats) {
			bytes += Profiler.GetRuntimeMemorySize(m);
		}
		
		var audio = Resources.FindObjectsOfTypeAll(typeof(AudioClip));
		foreach(AudioClip m in audio) {
			bytes += Profiler.GetRuntimeMemorySize(m);
		}
		
		var anims = Resources.FindObjectsOfTypeAll(typeof(Animation));
		foreach(Animation a in anims) {
			bytes += Profiler.GetRuntimeMemorySize(a);
		}
		
		Debug.Log ("total static res memory usage " + bytes/1024/1024 + " Mb");
		#endif

		Debug.Log ("mono heap size: " + Profiler.GetMonoHeapSize()/1024/1024);
		Debug.Log ("mono heap used: " + Profiler.GetMonoUsedSize()/1024/1024);
		Debug.Log ("total reserverd: " + Profiler.GetTotalReservedMemory()/1024/1024);
		Debug.Log ("total allocated: " + Profiler.GetTotalAllocatedMemory()/1024/1024);
		Debug.Log ("total unused reserve: " + Profiler.GetTotalUnusedReservedMemory()/1024/1024);
	}

	public static Vector3 GetPivotPoint(Mesh mesh)
	{
		Bounds bounds = mesh.bounds;
		return new Vector3(-bounds.min.x / bounds.size.x, 
		                   -bounds.min.y / bounds.size.y, 
		                   -bounds.min.z / bounds.size.z);
	}

	public static void SetPivotPoint(Mesh mesh, Vector3 pivot)
	{
		Bounds bounds = mesh.bounds;
		
		Vector3 currPivot = GetPivotPoint(mesh);
		Vector3 offset =  Vector3.Scale(pivot - currPivot, bounds.size);
		
		Vector3[] verts = mesh.vertices; 
		for(int i=0; i<verts.Length; i++) {
			verts[i] -= offset;
		}
		mesh.vertices = verts;
		mesh.RecalculateBounds();
	}

	public static void ScaleMesh(Mesh mesh, float scale)
	{
		Vector3[] verts = mesh.vertices; 
		for(int i=0; i<verts.Length; i++) {
			verts[i] = verts[i] * scale;
		}
		mesh.vertices = verts;
		mesh.RecalculateBounds();
	}
	
}
